import { GenericFriendlyError } from "../helpers/errors";
import { DynamoFilterQueryOperation } from "./dynamo-filter-query-operation";
import { DynamoQueryScanProcessor } from "./dynamo-query-scan-processor";
function applyMixins(derivedCtor, baseCtors) {
    baseCtors.forEach((baseCtor) => {
        Object.getOwnPropertyNames(baseCtor.prototype).forEach((name) => {
            Object.defineProperty(derivedCtor.prototype, name, Object.getOwnPropertyDescriptor(baseCtor.prototype, name));
        });
    });
}
export class DynamoFilterQueryMixin {
    allHelpValidateRequiredNumber(keyValueValidates) {
        const errors = [];
        Object.entries(keyValueValidates).forEach(([key, value]) => {
            if (!(!isNaN(Number(value)) && typeof value === "number")) {
                errors.push(`${key} is required`);
            }
        });
        if (errors.length) {
            throw new GenericFriendlyError(`${errors.join("; ")}.`);
        }
    }
    allHelpCreateFriendlyError(message, statusCode) {
        return new GenericFriendlyError(message);
    }
    allHelpValidateRequiredString(keyValueValidates) {
        const errors = [];
        Object.entries(keyValueValidates).forEach(([key, value]) => {
            if (!(value && typeof value === "string")) {
                errors.push(`${key} is required`);
            }
        });
        if (errors.length) {
            throw new GenericFriendlyError(`${errors.join("; ")}.`);
        }
    }
}
applyMixins(DynamoFilterQueryMixin, [
    DynamoFilterQueryOperation,
    DynamoQueryScanProcessor,
]);
export default DynamoFilterQueryMixin;
