/// <reference types="hapi__joi" />
import type { IDynamoQueryParamOptions, ISecondaryIndexDef, IFieldCondition, IDynamoQuerySecondayIndexOptions } from "../types";
import type { DynamoDB } from "aws-sdk";
import type { DocumentClient } from "aws-sdk/clients/dynamodb";
import Joi from "@hapi/joi";
import BaseMixins from "../core/base-mixins";
import { DynamoManageTable } from "./dynamo-manage-table";
interface IDynamoOptions<T> {
    dynamoDb: () => DynamoDB;
    schemaDef: Joi.SchemaMap;
    dynamoDbClient: () => DocumentClient;
    dataKeyGenerator: () => string;
    featureEntityValue: string;
    secondaryIndexOptions: ISecondaryIndexDef<T>[];
    baseTableName: string;
    strictRequiredFields: (keyof T)[] | string[];
}
export default abstract class DynamoDataOperation<T> extends BaseMixins {
    private readonly here_partitionKeyFieldName;
    private readonly here_sortKeyFieldName;
    private readonly here_dynamoDbClient;
    private readonly here_dynamoDb;
    private readonly here_dataKeyGenerator;
    private readonly here_schema;
    private readonly here_marshaller;
    private readonly here_tableFullName;
    private readonly here_strictRequiredFields;
    private readonly here_featureEntityValue;
    private readonly here_secondaryIndexOptions;
    private here_tableManager;
    constructor({ dynamoDb, schemaDef, dynamoDbClient, secondaryIndexOptions, featureEntityValue, baseTableName, strictRequiredFields, dataKeyGenerator, }: IDynamoOptions<T>);
    protected getTableManager(): DynamoManageTable<T>;
    private _dynamoDbClient;
    private _dynamoDb;
    private generateDynamoTableKey;
    private _getLocalVariables;
    private _getBaseObject;
    private _checkValidateMustBeAnObjectDataType;
    private _checkValidateStrictRequiredFields;
    protected allCreateOneBase({ data }: {
        data: T;
    }): Promise<T>;
    private withConditionPassed;
    protected allGetOneByIdBase({ dataId, withCondition, }: {
        dataId: string;
        withCondition?: IFieldCondition<T>;
    }): Promise<T | null>;
    protected allUpdateOneDirectBase({ data }: {
        data: T;
    }): Promise<T>;
    protected allUpdateOneByIdBase({ dataId, data, withCondition, }: {
        dataId: string;
        data: T;
        withCondition?: IFieldCondition<T>;
    }): Promise<T>;
    private _allHelpValidateMarshallAndGetValue;
    protected allQueryGetManyByConditionBase(paramOptions: IDynamoQueryParamOptions<T>): Promise<T[]>;
    protected allQueryGetManyByConditionPaginateBase(paramOptions: IDynamoQueryParamOptions<T>): Promise<import("../types").IDynamoPagingResult<T[]>>;
    protected allBatchGetManyByIdsBase({ dataIds, fields, withCondition, }: {
        dataIds: string[];
        fields?: (keyof T)[];
        withCondition?: IFieldCondition<T>;
    }): Promise<T[]>;
    protected allQuerySecondaryIndexBase<TData = T, TSortKeyField = string>(paramOption: IDynamoQuerySecondayIndexOptions<TData, TSortKeyField>): Promise<T[]>;
    protected allQuerySecondaryIndexPaginateBase<TData = T, TSortKeyField = string>(paramOption: IDynamoQuerySecondayIndexOptions<TData, TSortKeyField>): Promise<import("../types").IDynamoPagingResult<T[]>>;
    protected allDeleteByIdBase({ dataId, withCondition, }: {
        dataId: string;
        withCondition?: IFieldCondition<T>;
    }): Promise<T>;
    protected allDeleteManyDangerouselyByIds({ dataIds }: {
        dataIds: string[];
    }): Promise<boolean>;
}
export {};
